      subroutine meshdel (alayer, n1, n2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshdel  Updated     2005 March 21 14:00.
c               meshdel  Originated  1995 August 11 17:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To delete from the mesh layers of index type "alayer"
c               ("k", "l" or "m") with index values from n1 to n2.
c               All mesh layers of index type "alayer" with higher indices will
c               have their index reduced by the number of layers deleted, and
c               the size of the mesh will be adjusted accordingly.
c               Points with the deleted indices will be released from the mesh
c               ane renamed.
c               The index range of any mesh block may be changed.
c
c     Logic:    Entry points:  meshdel.
c meshdel       Called by:  mesh.
c
c meshdel       Calls:  aptchmv, meshsub, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     alayer    Input    Type of layer to delete from mesh, "k", "l" or "m".
c
c     n1        Input    Index of first layer of type "alayer" to delete.
c
c     n2        Input    Index of last layer of type "alayer" to delete.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input variables.

      character*8       alayer        ! Type of layer deleted, "k", "l" or "m".

c.... Local variables.

      character*8       adone         ! "RENAMED" or "REDUCED".

c.... Initialize.

      nprel  = 0
      npred  = 0
      nerr   = 0
      ndel   = n2 - n1 + 1
      adone  = ' '

 8110 format ('  meshdel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh layer type is not k, l or m:  ',a)

      nmax = 0
      if     (alayer .eq. 'k') then
        nmax = kmax
      elseif (alayer .eq. 'l') then
        nmax = lmax
      elseif (alayer .eq. 'm') then
        nmax = mmax
      else
        nerr  = 1
        write (   6, 8110) ainput, ninput, alayer
        write (iout, 8110) ainput, ninput, alayer
      endif

 8120 format ('  meshdel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh ',a1,' index N1 =',i6,' is <  1 or > N2 =',i6,'.')
 8130 format ('  meshdel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh ',a1,' index N2 =',i6,' is < N1 =',i6,
     &  ' or > NMAX =',i6,'.')

      if ((n1 .le. 0)   .or.
     &    (n1 .gt. n2)) then
        nerr = 1
        write (   6, 8120) ainput, ninput, alayer, n1, n2
        write (iout, 8120) ainput, ninput, alayer, n1, n2
      endif

      if ((n2 .lt. n1)     .or.
     &    (n2 .gt. nmax))  then
        nerr = 1
        write (   6, 8130) ainput, ninput, alayer, n2, n1, nmax
        write (iout, 8130) ainput, ninput, alayer, n2, n1, nmax
      endif

      if (nerr .ne. 0) go to 999

      newbl = 0                       ! Change to 1 if mesh block redefined.

c.... Save the old mesh size.

      kmaxold = kmax
      lmaxold = lmax
      mmaxold = mmax

      kbl1old = kbl1
      kbl2old = kbl2
      lbl1old = lbl1
      lbl2old = lbl2
      mbl1old = mbl1
      mbl2old = mbl2

c.... Release all points with "alayer" indices from n1 to n2,
c....   and reduce all higher "alayer" indices.

 8150 format ('  mesh point ',a,2x,3i6,7x,'! ',a)

      if (alayer .eq. 'k') then       ! Deleting k layers.

        kmax = kmax - ndel

        if (kmax .le. 0) then
          kmax = 0
          lmax = 0
          mmax = 0
        endif

        if (n1 .le. kbl2) then
          newbl = 1
        endif

        kbl1 = min0 (kbl1, max0 (n1,     kbl1 - ndel))
        kbl2 = min0 (kbl2, max0 (n1 - 1, kbl2 - ndel))
        if (kbl2 .lt. kbl1) then
          kbl1 = 0
          kbl2 = 0
          lbl1 = 0
          lbl2 = 0
          mbl1 = 0
          mbl2 = 0
        endif

        do 160 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 160
          if (kpoint(np) .lt. n1)  go to 160

          call pout (np, np, 'min')

          if (kpoint(np) .le. n2) then  ! Delete this point.

            adone      = 'RENAMED'
            nprel      = nprel + 1

            call aptchmv ('-', 1, 1, apoint(np),1, nerra)

            write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone
            write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone

            kpoint(np) = 0
            lpoint(np) = 0
            mpoint(np) = 0

          else                        ! Reduce the k index of this point.

            adone      = 'REDUCED'
            npred      = npred + 1
            kpoint(np) = kpoint(np) - ndel

            write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone
            write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone

          endif

          call meshsub (np, nrep, nerr)

 8160     format ('  meshdel ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Can not make point subscripts match mesh indices.')

          if (nerr .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

  160   continue                      ! End of loop over points.

      elseif (alayer .eq. 'l') then   ! Deleting l layers.

        lmax = lmax - ndel

        if (lmax .le. 0) then
          kmax = 0
          lmax = 0
          mmax = 0
        endif

        if (n1 .le. lbl2) newbl = 1

        lbl1 = min0 (lbl1, max0 (n1,     lbl1 - ndel))
        lbl2 = min0 (lbl2, max0 (n1 - 1, lbl2 - ndel))
        if (lbl2 .lt. lbl1) then
          kbl1 = 0
          kbl2 = 0
          lbl1 = 0
          lbl2 = 0
          mbl1 = 0
          mbl2 = 0
        endif

        do 170 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 170
          if (lpoint(np) .lt. n1)  go to 170

          call pout (np, np, 'min')

          if (lpoint(np) .le. n2) then  ! Delete this point.

            adone      = 'RENAMED'
            nprel      = nprel + 1

            call aptchmv ('-', 1, 1, apoint(np),1, nerra)

            write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone
            write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone

            kpoint(np) = 0
            lpoint(np) = 0
            mpoint(np) = 0

          else                        ! Reduce the k index of this point.

            adone      = 'REDUCED'
            npred      = npred + 1
            lpoint(np) = lpoint(np) - ndel

            write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone
            write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone

          endif

          call meshsub (np, nrep, nerr)

          if (nerr .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

  170   continue                      ! End of loop over points.

      elseif (alayer .eq. 'm') then   ! Deleting m layers.

        mmax = mmax - ndel

        if (mmax .le. 0) then
          kmax = 0
          lmax = 0
          mmax = 0
        endif

        if (n1 .le. mbl2) newbl = 1

        mbl1 = min0 (mbl1, max0 (n1,     mbl1 - ndel))
        mbl2 = min0 (mbl2, max0 (n1 - 1, mbl2 - ndel))
        if (mbl2 .lt. mbl1) then
          kbl1 = 0
          kbl2 = 0
          lbl1 = 0
          lbl2 = 0
          mbl1 = 0
          mbl2 = 0
        endif

        do 180 np = 1, npoints        ! Loop over points.
          if (apoint(np) .eq. ' ') go to 180
          if (mpoint(np) .lt. n1)  go to 180

          call pout (np, np, 'min')

          if (mpoint(np) .le. n2) then  ! Delete this point.

            adone      = 'RENAMED'
            nprel      = nprel + 1

            call aptchmv ('-', 1, 1, apoint(np),1, nerra)

            write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone
            write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone

            kpoint(np) = 0
            lpoint(np) = 0
            mpoint(np) = 0

          else                        ! Reduce the k index of this point.

            adone      = 'REDUCED'
            npred      = npred + 1
            mpoint(np) = mpoint(np) - ndel

            write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone
            write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np), adone

          endif

          call meshsub (np, nrep, nerr)

          if (nerr .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

  180   continue                      ! End of loop over points.

      endif                           ! Tested for k, l or m layer deletion.

c.... Display the number of points released and with mesh indices reduced.

 8210 format ('  Number of mesh points released and renamed =',i7 )
 8215 format ('  Number of mesh points with ',a1,
     &  ' indices reduced   =',i7 )

      if (nprel .gt. 0) then
        write (   6, 8210) nprel
        write (iout, 8210) nprel
      endif

      if (npred .gt. 0) then
        write (   6, 8215) alayer, npred
        write (iout, 8215) alayer, npred
      endif

c.... Display the old and new mesh sizes.

 8140 format ('  WARNING:  changing old mesh size.' /
     &  'Mesh size:   kmax old  =',i6,'   lmax old  =',i6,
     &  '   mmax old  =',i6 )
 8220 format (
     &  'Mesh size:   kmax      =',i6,'   lmax      =',i6,
     &  '   mmax      =',i6 /
     &  '             kmax*lmax =',i6,'   lmax*mmax =',i6,
     &  '   mmax*kmax =',i6 /
     &  '             kmax*lmax*mmax                =',i6)

      write (   6, 8140) kmaxold, lmaxold, mmaxold
      write (iout, 8140) kmaxold, lmaxold, mmaxold

      klmax   = kmax * lmax
      lmmax   = lmax * mmax
      mkmax   = mmax * lmax
      klmmax  = kmax * lmax * mmax

      write (   6, 8220) kmax, lmax, mmax,
     &  klmax, lmmax, mkmax, klmmax
      write (iout, 8220) kmax, lmax, mmax,
     &  klmax, lmmax, mkmax, klmmax

c.... Display the new mesh block, if it was changed.

 8230 format ('  WARNING:  changing old mesh block indices:' /
     &  'Mesh block:  kbl1 old  =',i6,'   lbl1 old  =',i6,
     &  '   mbl1 old  =',i6 /
     &  '             kbl2 old  =',i6,'   lbl2 old  =',i6,
     &  '   mbl2 old  =',i6 )
 8240 format (
     &  'Mesh block:  kbl1      =',i6,'   lbl1      =',i6,
     &  '   mbl1      =',i6 /
     &  '             kbl2      =',i6,'   lbl2      =',i6,
     &  '   mbl2      =',i6 )

      if (newbl .eq. 1) then
        write (   6, 8230) kbl1old, lbl1old, mbl1old,
     &                     kbl2old, lbl2old, mbl2old
        write (iout, 8230) kbl1old, lbl1old, mbl1old,
     &                     kbl2old, lbl2old, mbl2old
        write (   6, 8240) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8240) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
      endif

  999 return

c.... End of subroutine meshdel.      (+1 line)
      end
