      subroutine track

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  track    Updated     2007 January 19 15:30.
c               track    Originated  1993 November 24 12:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the distance from the beginning of a linear track to
c               the intersection with, or proximal point to, a point, a line,
c               a triangle, a plane, a sphere, a circular cylinder, a circular
c               cone, a hyperbolic paraboloid, an ellipsoid, an axisymmetric
c               quadric surface, a general quadric surface, or a zone.
c               For an intersection, to find the angle of incidence, and if from
c               a surface, the reflection direction.
c
c     Logic:    Entry points:  track.
c track         Called by:  helpall, main.
c
c track         Calls:  aptlnln, aptlnpl, aptptln, aptptpl, aptrefs, aptrkis,
c               aptvang, aptvdis, aptvdot, aptvpln, aptvusz, coord, lout,
c               plotpt, plout, ptfind, quout, qutype, trout, vectfind, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /cctarg/ asysx           ! Coordinate system.
      character*8     asysx           ! Coordinate system.
      common /cctarg/ ageomt          ! Geometry type of target.
      character*8     ageomt          ! Geometry type of target.
      common /cctarg/ refm(3,3)       ! Reflection tensor.

      character*8 amount              ! Amount of output data.

c===============================================================================

c.... Input line:  track [help]

 8110 format ('  Command "track":  acceptable input lines include:' /
     &  '  "track [help]"  Display this message.' //
     &  '  "track PNAME VNAME point PNAME2"' /
     &  '     Find the distance to and coordinates of the point on' /
     &  '     the track through point PNAME in direction VNAME,' /
     &  '     nearest to point PNAME2.' //
     &  '  "track PNAME VNAME line ALNAME"' /
     &  '     Find the distance to and coordinates of the point on' /
     &  '     the track through point PNAME in direction VNAME,' /
     &  '     nearest to line ALNAME.' //
     &  '  "track PNAME VNAME triangle TRNAME"' /
     &  '     Find the distance to and coordinates of the point on' /
     &  '     the track through point PNAME in direction VNAME,' /
     &  '     intersecting triangle TRNAME.' //
     &  '  "track PNAME VNAME QTYPE QNAME"' /
     &  '     Find the distance to and coordinates of the point on' /
     &  '     the track through point PNAME in direction VNAME,' /
     &  '     intersecting the quadric surface of type QTYPE with name',
     &  ' QNAME.' )

 8112 format (
     &  '     QTYPE must be plane, sphere, cylinder, cone, hyperb,',
     &  ' ellipsoid, axisym,' /
     &  '     or quadric.' //
     &  '  "track PNAME VNAME zone ZNAME"' /
     &  '     Find the distances to and coordinates of the points' /
     &  '     on the track through point PNAME in direction VNAME,' /
     &  '     intersecting the surfaces bounding zone ZNAME.' //
     &  '  Synonyms:  [cylinder, cyl], [ellipsoid, ellipse, ell, el],',
     &  ' [help, h],' /
     &  '    [hyperb, saddle], [line, ln, l], [plane, pl],' /
     &  '    [point, pnt, pt, p], [quadric, quad, q],' /
     &  '    [sphere, sph], [track, trk], [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        go to 999
      endif

c===============================================================================

c.... Initialize.

      pdist   = -1.e99

      proxptx = -1.e99
      proxpty = -1.e99
      proxptz = -1.e99

      proxvx  = -1.e99
      proxvy  = -1.e99
      proxvz  = -1.e99

c===============================================================================

c.... Test input for validity.

      nerr = 0

 8120 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on input line not 5:  ',i3)

      if (nword .ne. 5) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c===============================================================================

c.... Find the point at the beginning of the track.

      call ptfind (2, npbeg)

      if (npbeg .eq. 0) then
        go to 999
      endif

      xbeg = x(npbeg)
      ybeg = y(npbeg)
      zbeg = z(npbeg)

c===============================================================================

c.... Find the vector along the track.

 8135 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Direction vector has zero length:  ',a)
 8140 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Direction vector has zero length relative to the',
     &  ' point:  ',a)

c.... See if the vector exists.

      call vectfind (3, nv)

      if (nv .eq. 0) then
        go to 999
      endif

c.... See if the vector length is zero.

      if (vlen(nv) .eq. 0.0) then
        write (   6, 8135) ainput, ninput, aword(3)
        write (iout, 8135) ainput, ninput, aword(3)
        go to 999
      endif

c.... See if the vector length is small relative to the point coordinates.

      xend = xbeg + xcom(nv)
      yend = ybeg + ycom(nv)
      zend = zbeg + zcom(nv)

      call aptvdis (xbeg, ybeg, zbeg, xend, yend, zend, 1, tol,
     &              dx, dy, dz, dlen, nerra)

      if (dlen .eq. 0.0) then
        write (   6, 8140) ainput, ninput, aword(3)
        write (iout, 8140) ainput, ninput, aword(3)
        go to 999
      endif

c===============================================================================

c.... Find the target object.

    5 ageomt = ' '

c===============================================================================

c.... Input line:  track PNAME VNAME point     PNAME2

      if ((aword(4) .eq. 'point') .or.
     &    (aword(4) .eq. 'pnt')   .or.
     &    (aword(4) .eq. 'pt')    .or.
     &    (aword(4) .eq. 'p'))    then

        call ptfind (5, nt)

        if (nt .eq. 0) then
          go to 310
        endif

        ageomt = 'point'

        go to 310

      endif

c===============================================================================

c.... Input line:  track PNAME VNAME line      ALNAME

 8210 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named line does not exist:  ',a)
 8220 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Line has zero length:  ',a )

      if ((aword(4) .eq. 'line') .or.
     &    (aword(4) .eq. 'ln')   .or.
     &    (aword(4) .eq. 'l'))  then

c....   Find the line.

        ageomt = 'line'

        do 210 n = 1, nlines
          if (aword(5) .eq. aline(n)) then
            nt  = n
            call lout (nt, nt)
            go to 212
          endif
  210   continue
        write (   6, 8210) ainput, ninput, aword(5)
        write (iout, 8210) ainput, ninput, aword(5)
        go to 310

c....   Find the end points of the line.

  212   do 220 np = 1, npoints
          if (alipt1(nt) .eq. apoint(np)) then
            n1 = np
            go to 222
          endif
  220   continue
        write (   6, 8220) ainput, ninput, alipt1(nt)
        write (iout, 8220) ainput, ninput, alipt1(nt)
        go to 310

  222   do 230 np = 1, npoints
          if (alipt2(nt) .eq. apoint(np)) then
            n2 = np
            go to 232
          endif
  230   continue
        write (   6, 8220) ainput, ninput, alipt2(nt)
        write (iout, 8220) ainput, ninput, alipt2(nt)
        go to 310

c....   See if the line has a zero length.

  232   call aptvdis (x(n1), y(n1), z(n1), x(n2), y(n2), z(n2), 1, tol,
     &                dlinex, dliney, dlinez, dlen, nerra)

        if (dlen .eq. 0.0) then
          write (   6, 8220) ainput, ninput, aword(5)
          write (iout, 8220) ainput, ninput, aword(5)
          go to 310
        endif
      endif

c===============================================================================

c.... Input line:  track PNAME VNAME triangle  TRNAME

 8240 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named triangle does not exist:  ',a)
 8250 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Triangle is indeterminate:  ',a )
 8241 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Vertex point does not exist:  ',a)

      if ((aword(4) .eq. 'triangle') .or.
     &    (aword(4) .eq. 'tri'))   then

        ageomt = 'triangle'

        do 240 n = 1, ntrigs
          if (aword(5) .eq. atrig(n)) then
            nt  = n

            do 241 np = 1, npoints
              if (apoint(np) .eq. atript1(nt)) then
                xtra = x(np)
                ytra = y(np)
                ztra = z(np)
                go to 242
              endif
  241       continue
            write (   6, 8241) ainput, ninput, atript1(nt)
            write (iout, 8241) ainput, ninput, atript1(nt)
            go to 310

  242       do 243 np = 1, npoints
              if (apoint(np) .eq. atript2(nt)) then
                xtrb = x(np)
                ytrb = y(np)
                ztrb = z(np)
                go to 244
              endif
  243       continue
            write (   6, 8241) ainput, ninput, atript2(nt)
            write (iout, 8241) ainput, ninput, atript2(nt)
            go to 310

  244       do 245 np = 1, npoints
              if (apoint(np) .eq. atript3(nt)) then
                xtrc = x(np)
                ytrc = y(np)
                ztrc = z(np)
                go to 246
              endif
  245       continue
            write (   6, 8241) ainput, ninput, atript3(nt)
            write (iout, 8241) ainput, ninput, atript3(nt)
            go to 310

  246       amount = 'maximum'

            call trout (nt, nt, amount)
            go to 250
          endif
  240   continue
        write (   6, 8240) ainput, ninput, aword(5)
        write (iout, 8240) ainput, ninput, aword(5)
        go to 310

c....   See if the triangle is indeterminate.

c....   Find the normal vector to the triangle, see if it is zero.

  250   call aptvpln (xtra, ytra, ztra, xtrb, ytrb, ztrb,
     &                xtrc, ytrc, ztrc, 1, tol,
     &                xn, yn, zn, vlength, nerra)

        if (vlength .eq. 0.0) then
          write (   6, 8250) ainput, ninput, aword(5)
          write (iout, 8250) ainput, ninput, aword(5)
          go to 310
        endif

      endif

c===============================================================================

c.... Input line:  track PNAME VNAME plane     PLNAME

 8270 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named plane does not exist:  ',a)
 8280 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Plane is indeterminate:  ',a )

      if ((aword(4) .eq. 'plane') .or.
     &    (aword(4) .eq. 'pl'))   then

c....   Find the plane.

        ageomt = 'plane'

        do 270 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(0)) go to 270
          if (aword(5) .eq. aquad(n)) then
            nq  = n
            call plout (nq, nq)
            go to 280
          endif
  270   continue
        write (   6, 8270) ainput, ninput, aword(5)
        write (iout, 8270) ainput, ninput, aword(5)
        go to 310

c....   See if the plane is indeterminate.

  280   xn = qx(nq)
        yn = qy(nq)
        zn = qz(nq)

        call aptvusz (xn, yn, zn, tol, dx, dy, dz, vlength)

        if (vlength .eq. 0.0) then
          write (   6, 8280) ainput, ninput, aword(5)
          write (iout, 8280) ainput, ninput, aword(5)
          go to 310
        endif

      endif

c===============================================================================

c.... Input line:  track PNAME VNAME axisym    AXNAME
c.... Input line:  track PNAME VNAME cone      CNNAME
c.... Input line:  track PNAME VNAME cylinder  CYLNAME
c.... Input line:  track PNAME VNAME ellipsoid ELNAME
c.... Input line:  track PNAME VNAME hyperb    HPNAME
c.... Input line:  track PNAME VNAME quadric   QNAME
c.... Input line:  track PNAME VNAME sphere    SPHNAME

 8290 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)
 8295 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric surface has bad coefficients:  ',a )

      if ((aword(4) .eq. 'quadric')   .or.
     &    (aword(4) .eq. 'quad')      .or.
     &    (aword(4) .eq. 'q')         .or.
     &    (aword(4) .eq. 'sphere')    .or.
     &    (aword(4) .eq. 'sph')       .or.
     &    (aword(4) .eq. 'cylinder')  .or.
     &    (aword(4) .eq. 'cyl')       .or.
     &    (aword(4) .eq. 'cone')      .or.
     &    (aword(4) .eq. 'hyperb')    .or.
     &    (aword(4) .eq. 'saddle')    .or.
     &    (aword(4) .eq. 'ellipsoid') .or.
     &    (aword(4) .eq. 'ellipse')   .or.
     &    (aword(4) .eq. 'ell')       .or.
     &    (aword(4) .eq. 'el')        .or.
     &    (aword(4) .eq. 'axisym'))   then

        ageomt = 'quadric'

        do 290 n = 1, nquads
          if (aword(5) .eq. aquad(n)) then

            call qutype (aword(4), n, 1, nerrq)

            if (nerrq .ne. 0) go to 999

            nq  = n
            call quout (nq, nq, 'min', nerrq)
            go to 295
          endif
  290   continue
        write (   6, 8290) ainput, ninput, aword(5)
        write (iout, 8290) ainput, ninput, aword(5)
        go to 310

c....   See if the quadric is poorly defined.

  295   if (nerrq .ne. 0.0) then
          write (   6, 8295) ainput, ninput, aword(5)
          write (iout, 8295) ainput, ninput, aword(5)
          go to 310
        endif

      endif

c===============================================================================

c.... Input line:  track PNAME VNAME zone      ZNAME

 8296 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named zone does not exist:  ',a)

      if ((aword(4) .eq. 'zone') .or.
     &    (aword(4) .eq. 'zn')   .or.
     &    (aword(4) .eq. 'z'))   then

        ageomt = 'zone'

        do 298 n = 1, nzones
          if (aword(5) .eq. azone(n)) then
            nz  = n
            call zout (nz, nz, 'min')
            go to 299
          endif
  298   continue
        write (   6, 8296) ainput, ninput, aword(5)
        write (iout, 8296) ainput, ninput, aword(5)
        go to 310

  299   continue

      endif

c===============================================================================

c.... Input line:  unrecognized.

 8310 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Method not recognized:  object ',a)

  310 if (ageomt .eq. ' ') then
        write (   6, 8310) ainput, ninput, aword(4)
        write (iout, 8310) ainput, ninput, aword(4)
        go to 999
      endif

      if (nerr .ne. 0) go to 999

c.... Switch to cartesian coordinates, if necessary.

      asysx = asys
      if (asysx .ne. 'rect') then
        nword = 2
        aword(2) = 'rect'
        call coord
      endif

c===============================================================================

      if (ageomt .ne. 'point') go to 380

c.... Input line:  track PNAME VNAME point     PNAME2

c.... Find the distance along the track, and the coordinates of the point
c....   intersecting or nearest the target point.

 8320 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not find distance from track of zero length.')
 8330 format ('Point is on track ',1p3e20.12)
 8340 format ('Prox pt trk  (xyz)',1p3e20.12)
 8345 format ('Prox pt line (xyz)',1p3e20.12)
 8350 format ('Proximal distance ',1pe20.12)
 8360 format ('  Track distance  ',1pe20.12)

c.... Find the proximal point on the track, and the distance from the proximal
c....   point to the target point.

      call aptptln (x(nt), y(nt), z(nt),
     &              xbeg, ybeg, zbeg, xend, yend, zend,
     &              1, tol, 0, dpmin, fdmin, cx, cy, cz,
     &              nlim, itrun, nerra)

c.... Find the distance from the beginning of the track to the proximal point.

      call aptvdis (xbeg, ybeg, zbeg, cx, cy, cz, 1, tol,
     &              dx, dy, dz, dtrack, nerra)

c.... Find right sign of track distance to the proximal point.

      call aptvdot (dx, dy, dz, xcom(nv), ycom(nv), zcom(nv), 1, tol,
     &              spab, nerra)

      if (spab .lt. 0.0) dtrack = -dtrack

      if (itrun .eq. -1) then

        write (   6, 8320) ainput, ninput
        write (iout, 8320) ainput, ninput
        go to 710

      elseif (dpmin .eq. 0.0) then

        pdist = 0.0

        write (   6, 8330) x(nt), y(nt), z(nt)
        write (iout, 8330) x(nt), y(nt), z(nt)
        call plotpt (x(nt), y(nt), z(nt), nerrg)

        proxptx = x(nt)
        proxpty = y(nt)
        proxptz = z(nt)

        proxvx  = 0.0
        proxvy  = 0.0
        proxvz  = 0.0

      else

        pdist = dpmin

        write (   6, 8340) cx, cy, cz ! Proximal point.
        write (iout, 8340) cx, cy, cz
        call plotpt (cx, cy, cz, nerrg)

        proxptx = cx
        proxpty = cy
        proxptz = cz

        proxvx  = x(nt) - cx
        proxvy  = y(nt) - cy
        proxvz  = z(nt) - cz

        write (   6, 8350) dpmin      ! Distance from proximal pt to target pt.
        write (iout, 8350) dpmin

      endif

      write (   6, 8360) dtrack       ! Track distance to proximal point.
      write (iout, 8360) dtrack

      go to 710

c===============================================================================

  380 if (ageomt .ne. 'line') go to 450

c.... Input line:  track PNAME VNAME line      ALNAME

c.... Find the distance along the track, and the coordinates of the point
c....   intersecting or nearest the target line.

 8400 format ('Intersection at   ',1p3e20.12)
 8440 format ('Line is parallel  ')

c.... Find the proximal point on the track, and the distance from the proximal
c....   point to the target line.

      call aptlnln (xbeg, ybeg, zbeg, xend, yend, zend,
     &              x(n1), y(n1), z(n1), x(n2), y(n2), z(n2),
     &              1, tol, dpmin, fracab, fraccd, ex, ey, ez,
     &              fx, fy, fz, itrun, ipar, nerra)

c.... Find the distance from the beginning of the track to the proximal point.

      call aptvdis (xbeg, ybeg, zbeg, ex, ey, ez, 1, tol,
     &              dx, dy, dz, dtrack, nerra)

c.... Find the right sign of the track distance to the proximal point.

      call aptvdot (dx, dy, dz, xcom(nv), ycom(nv), zcom(nv), 1, tol,
     &              spab, nerra)

      if (spab .lt. 0.0) dtrack = -dtrack

c.... Print results.


      if (dpmin .eq. 0.0) then        ! Track and line intersect.

        pdist = 0.0

        write (   6, 8400) ex, ey, ez ! Intersection point.
        write (iout, 8400) ex, ey, ez
        call plotpt (ex, ey, ez, nerrg)

        proxptx = ex
        proxpty = ey
        proxptz = ez

        proxvx  = 0.0
        proxvy  = 0.0
        proxvz  = 0.0

      else                            ! Track and line do not intersect.

        pdist = dpmin

        write (   6, 8340) ex, ey, ez ! Proximal point on track.
        write (iout, 8340) ex, ey, ez
        call plotpt (ex, ey, ez, nerrg)

        write (   6, 8345) fx, fy, fz ! Proximal point on line.
        write (iout, 8345) fx, fy, fz
        call plotpt (fx, fy, fz, nerrg)

        proxptx = ex
        proxpty = ey
        proxptz = ez

        proxvx  = fx - ex
        proxvy  = fy - ey
        proxvz  = fz - ez

        write (   6, 8350) dpmin      ! Distance from proximal pt to target pt.
        write (iout, 8350) dpmin

      endif                           ! Tested for intersection.

      if (ipar .eq. 0) then
        write (   6, 8360) dtrack      ! Track distance to proximal point.
        write (iout, 8360) dtrack
      else
        write (   6, 8440)            ! Line is parallel to track.
        write (iout, 8440)
      endif

      if (dpmin .eq. 0.0) then        ! An intersection occurred.

c....   Find the cosine of the angle of intersection.

        call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                dlinex, dliney, dlinez, 1, tol,
     &                vcos, nerra)

        if (dtrack .lt. 0.0) vcos = -vcos

c....   Find the angle of intersection.

        vang = acos (vcos)
        if (abs (vang) .lt. tol) vang = 0.0
        if (abs (vang - pi) .lt. tol) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

 8380   format ('  Cosine of angle ',1pe20.12,'  (from line)' /
     &          '  Angle (',a,')',1pe20.12,'  (from line)' )

        write (   6, 8380) vcos, angunit, vang
        write (iout, 8380) vcos, angunit, vang
      endif

      go to 710

c===============================================================================

  450 if (ageomt .ne. 'triangle') go to 480

c.... Input line:  track PNAME VNAME triangle  TRNAME

c.... Find the distance along the track, and the coordinates of the point
c....   intersecting or nearest the target triangle.

 8480 format ('Parallel distance ',1pe20.12)

c.... Find the intersection point on the track, and the distance from the
c....   beginning of the track to the intersection point.

      call aptlnpl (xbeg, ybeg, zbeg, xend, yend, zend,
     &              xtra, ytra, ztra, xn, yn, zn, 1, tol,
     &              dpmin, dtrack, fracab, xint, yint, zint,
     &              ipar, nerra)

      if (ipar .eq. 0) then           ! Not parallel.

        pdist = 0.0

        write (   6, 8400) xint, yint, zint  ! Intersection point.
        write (iout, 8400) xint, yint, zint
        call plotpt (xint, yint, zint, nerrg)

        write (   6, 8360) dtrack     ! Track distance to intersection.
        write (iout, 8360) dtrack

c....   Find the cosine of the angle of intersection.

        call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                xn, yn, zn, 1, tol, vcos, nerra)

        if (dtrack .lt. 0.0) vcos = -vcos

c....   Find the angle of intersection.

        vang = acos (vcos)
        if (abs (vang) .lt. 1.e-5) vang = 0.0
        if (abs (vang - pi) .lt. 1.e-5) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

 8535   format ('  Can not find the angle if one vector has',
     &    ' zero length.')
 8536   format ('  Cosine of angle ',1pe20.12,'  (from normal)' /
     &          '  Angle (',a,')',1pe20.12,'  (from normal)' )

        write (   6, 8536) vcos, angunit, vang
        write (iout, 8536) vcos, angunit, vang

c....   Find the reflected track vector.

        xref = xcom(nv)
        yref = ycom(nv)
        zref = zcom(nv)

        call aptrefs (0, 0., 0., 0., xn, yn, zn, 0., 0., 0.,
     &                xref, yref, zref, 1, tol, refm, nerra)

 8538   format ('  Refl vect (xyz) ',1p3e20.12)

        write (   6, 8538) xref, yref, zref
        write (iout, 8538) xref, yref, zref

        proxptx = xint
        proxpty = yint
        proxptz = zint

        proxvx  = 0.0
        proxvy  = 0.0
        proxvz  = 0.0

      else                            ! Track and triangle are parallel.

        pdist = dpmin

        write (   6, 8480) dpmin      ! Parallel distance.
        write (iout, 8480) dpmin

        call aptptpl (xbeg, ybeg, zbeg,
     &                xtra, ytra, ztra, xn, yn, zn, 1, tol,
     &                dpmin, cx, cy, cz, itrun, nerr)

        proxptx = xbeg
        proxpty = ybeg
        proxptz = zbeg

        proxvx  = cx - xbeg
        proxvx  = cy - ybeg
        proxvz  = cz - zbeg

      endif

      go to 710

c===============================================================================

  480 if (ageomt .ne. 'plane') go to 510

c.... Input line:  track PNAME VNAME plane     PLNAME

c.... Find the distance along the track, and the coordinates of the point
c....   intersecting or nearest the target plane.

c.... Find the intersection point on the track, and the distance from the
c....   beginning of the track to the intersection point.

      call aptlnpl (xbeg, ybeg, zbeg, xend, yend, zend,
     &              xcen(nq), ycen(nq), zcen(nq),
     &              xn, yn, zn, 1, tol,
     &              dpmin, dtrack, fracab, xint, yint, zint,
     &              ipar, nerra)

      if (ipar .eq. 0) then           ! Not parallel.

        pdist = 0.0

        write (   6, 8400) xint, yint, zint  ! Intersection point.
        write (iout, 8400) xint, yint, zint
        call plotpt (xint, yint, zint, nerrg)

        write (   6, 8360) dtrack     ! Track distance to intersection.
        write (iout, 8360) dtrack

c....   Find the cosine of the angle of intersection.

        call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                xn, yn, zn, 1, tol, vcos, nerra)

        if (dtrack .lt. 0.0) vcos = -vcos

c....   Find the angle of intersection.

        vang = acos (vcos)
        if (abs (vang) .lt. 1.e-5) vang = 0.0
        if (abs (vang - pi) .lt. 1.e-5) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

        write (   6, 8536) vcos, angunit, vang
        write (iout, 8536) vcos, angunit, vang

c....   Find the reflected track vector.

        xref = xcom(nv)
        yref = ycom(nv)
        zref = zcom(nv)

        call aptrefs (0, 0., 0., 0., xn, yn, zn, 0., 0., 0.,
     &                xref, yref, zref, 1, tol, refm, nerra)


        write (   6, 8538) xref, yref, zref
        write (iout, 8538) xref, yref, zref

        proxptx = xint
        proxpty = yint
        proxptz = zint

        proxvx  = 0.0
        proxvy  = 0.0
        proxvz  = 0.0

      else                            ! Track parallel to plane.

        pdist = dpmin

        write (   6, 8480) dpmin      ! Parallel distance.
        write (iout, 8480) dpmin

        call aptptpl (xbeg, ybeg, zbeg,
     &                xcen(nq), ycen(nq), zcen(nq), xn, yn, zn, 1, tol,
     &                dpmin, cx, cy, cz, itrun, nerr)

        proxptx = xbeg
        proxpty = ybeg
        proxptz = zbeg

        proxvx  = cx - xbeg
        proxvx  = cy - ybeg
        proxvz  = cz - zbeg

      endif

      go to 710

c===============================================================================

  510 if (ageomt .ne. 'quadric') go to 610

c.... Input line:  track PNAME VNAME axisymc   QNAME
c.... Input line:  track PNAME VNAME cone      QNAME
c.... Input line:  track PNAME VNAME cylinder  QNAME
c.... Input line:  track PNAME VNAME ellipsoid QNAME
c.... Input line:  track PNAME VNAME hyperb    QNAME
c.... Input line:  track PNAME VNAME quadric   QNAME
c.... Input line:  track PNAME VNAME sphere    QNAME

c.... Find the distance along the track, and the coordinates of the point
c....   intersecting or nearest the target quadric.

c.... Find the intersection point on the track, and the distance from the
c....   beginning of the track to the intersection point.

      tolp = tol * (abs (xbeg) + abs (ybeg) + abs (zbeg))
      if (tolp .eq. 0.0) tolp = tol
      nints = 0

 8532 format ('No intersection in negative direction.')

      dintmn = -1.e99
      dintmx =  -tolp

      call aptrkis (xbeg, ybeg, zbeg, xcom(nv), ycom(nv), zcom(nv),
     &              qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq),
     &              dintmn, dintmx, 1, tol,
     &              nint, xint, yint, zint, dtrack,
     &              snx, sny, snz, costh, nerra)

      if (nint .eq. 1) then           ! One intersection of track with quadric.

        nints = nints + 1
        write (   6, 8400) xint, yint, zint  ! Intersection point.
        write (iout, 8400) xint, yint, zint
        call plotpt (xint, yint, zint, nerrg)

        write (   6, 8360) dtrack     ! Track distance to intersection.
        write (iout, 8360) dtrack

c....   Find the normal vector at the intersection point.

        xnorm = qx(nq) + 2.0 * qxx(nq) * xint + qxy(nq) * yint +
     &          qzx(nq) * zint
        ynorm = qy(nq) + qxy(nq) * xint + 2.0 * qyy(nq) * yint +
     &          qyz(nq) * zint
        znorm = qz(nq) + qzx(nq) * xint + qyz(nq) * yint +
     &          2.0 * qzz(nq) * zint

 8534   format ('  Norm vect (xyz) ',1p3e20.12)

        write (   6, 8534) xnorm, ynorm, znorm
        write (iout, 8534) xnorm, ynorm, znorm

        call aptvusz (xnorm, ynorm, znorm, tol, ux, uy, uz, vlength)

c....   Find the cosine of the angle of intersection.

        call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                ux, uy, uz, 1, tol, vcos, nerra)

        if (dtrack .lt. 0.0) vcos = -vcos

c....   Find the angle of intersection.

        vang = acos (vcos)
        if (abs (vang) .lt. 1.e-5) vang = 0.0
        if (abs (vang - pi) .lt. 1.e-5) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

        if (vlength .eq. 0.0) then
          write (   6, 8535)
          write (iout, 8535)
          go to 520
        endif

        write (   6, 8536) vcos, angunit, vang
        write (iout, 8536) vcos, angunit, vang

c....   Find the reflected track vector.

        xref = xcom(nv)
        yref = ycom(nv)
        zref = zcom(nv)

        call aptrefs (0, 0., 0., 0., xnorm, ynorm, znorm, 0., 0., 0.,
     &                xref, yref, zref, 1, tol, refm, nerra)


        write (   6, 8538) xref, yref, zref
        write (iout, 8538) xref, yref, zref

        pdist = 0.0

        proxptx = xint
        proxpty = yint
        proxptz = zint

        proxvx  = 0.0
        proxvy  = 0.0
        proxvz  = 0.0

      elseif (nint .eq. 0) then       ! No intersection.

        write (   6, 8532)
        write (iout, 8532)

      else                            ! Vector is too short.

        write (   6, 8135)  ainput, ninput, aword(3)
        write (iout, 8135)  ainput, ninput, aword(3)

      endif                           ! Tested number of intersections.

 8542 format ('No intersection near initial point.')

  520 dintmn = -tolp
      dintmx =  tolp

      call aptrkis (xbeg, ybeg, zbeg, xcom(nv), ycom(nv), zcom(nv),
     &              qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq),
     &              dintmn, dintmx, 1, tol,
     &              nint, xint, yint, zint, dtrack,
     &              snx, sny, snz, costh, nerra)

      if (nint .eq. 1) then           ! One intersection of track with quadric.

        nints = nints + 1
        write (   6, 8400) xint, yint, zint  ! Intersection point.
        write (iout, 8400) xint, yint, zint
        call plotpt (xint, yint, zint, nerrg)

        write (   6, 8360) dtrack     ! Track distance to intersection.
        write (iout, 8360) dtrack

c....   Find the normal vector at the intersection point.

        xnorm = qx(nq) + 2.0 * qxx(nq) * xint + qxy(nq) * yint +
     &          qzx(nq) * zint
        ynorm = qy(nq) + qxy(nq) * xint + 2.0 * qyy(nq) * yint +
     &          qyz(nq) * zint
        znorm = qz(nq) + qzx(nq) * xint + qyz(nq) * yint +
     &          2.0 * qzz(nq) * zint

        write (   6, 8534) xnorm, ynorm, znorm
        write (iout, 8534) xnorm, ynorm, znorm

        call aptvusz (xnorm, ynorm, znorm, tol, ux, uy, uz, vlength)

c....   Find the cosine of the angle of intersection.

        call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                ux, uy, uz, 1, tol, vcos, nerra)

        if (dtrack .lt. 0.0) vcos = -vcos

c....   Find the angle of intersection.

        vang = acos (vcos)
        if (abs (vang) .lt. 1.e-5) vang = 0.0
        if (abs (vang - pi) .lt. 1.e-5) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

        if (vlength .eq. 0.0) then
          write (   6, 8535)
          write (iout, 8535)
          go to 530
        endif

        write (   6, 8536) vcos, angunit, vang
        write (iout, 8536) vcos, angunit, vang

c....   Find the reflected track vector.

        xref = xcom(nv)
        yref = ycom(nv)
        zref = zcom(nv)

        call aptrefs (0, 0., 0., 0., xnorm, ynorm, znorm, 0., 0., 0.,
     &                xref, yref, zref, 1, tol, refm, nerra)

        write (   6, 8538) xref, yref, zref
        write (iout, 8538) xref, yref, zref

        pdist = 0.0

        proxptx = xint
        proxpty = yint
        proxptz = zint

        proxvx  = 0.0
        proxvy  = 0.0
        proxvz  = 0.0

      elseif (nint .eq. 0) then       ! No intersection.

        write (   6, 8542)
        write (iout, 8542)

      else                            ! Vector is too short.

        write (   6, 8135)  ainput, ninput, aword(3)
        write (iout, 8135)  ainput, ninput, aword(3)

      endif                           ! Tested number of intersections.

 8545 format ('No intersection in positive direction.')

  530 dintmn = tolp
      dintmx = 1.e99

      call aptrkis (xbeg, ybeg, zbeg, xcom(nv), ycom(nv), zcom(nv),
     &              qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq),
     &              dintmn, dintmx, 1, tol,
     &              nint, xint, yint, zint, dtrack,
     &              snx, sny, snz, costh, nerra)

      if (nint .eq. 1) then           ! One intersection of track with quadric.

        nints = nints + 1
        write (   6, 8400) xint, yint, zint  ! Intersection point.
        write (iout, 8400) xint, yint, zint
        call plotpt (xint, yint, zint, nerrg)

        write (   6, 8360) dtrack     ! Track distance to intersection.
        write (iout, 8360) dtrack

c....   Find the normal vector at the intersection point.

        xnorm = qx(nq) + 2.0 * qxx(nq) * xint + qxy(nq) * yint +
     &          qzx(nq) * zint
        ynorm = qy(nq) + qxy(nq) * xint + 2.0 * qyy(nq) * yint +
     &          qyz(nq) * zint
        znorm = qz(nq) + qzx(nq) * xint + qyz(nq) * yint +
     &          2.0 * qzz(nq) * zint

        write (   6, 8534) xnorm, ynorm, znorm
        write (iout, 8534) xnorm, ynorm, znorm

        call aptvusz (xnorm, ynorm, znorm, tol, ux, uy, uz, vlength)

c....   Find the cosine of the angle of intersection.

        call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                ux, uy, uz, 1, tol, vcos, nerra)

        if (dtrack .lt. 0.0) vcos = -vcos

c....   Find the angle of intersection.

        vang = acos (vcos)
        if (abs (vang) .lt. 1.e-5) vang = 0.0
        if (abs (vang - pi) .lt. 1.e-5) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

        if (vlength .eq. 0.0) then
          write (   6, 8535)
          write (iout, 8535)
          go to 540
        endif

        write (   6, 8536) vcos, angunit, vang
        write (iout, 8536) vcos, angunit, vang

c....   Find the reflected track vector.

        xref = xcom(nv)
        yref = ycom(nv)
        zref = zcom(nv)

        call aptrefs (0, 0., 0., 0., xnorm, ynorm, znorm, 0., 0., 0.,
     &                xref, yref, zref, 1, tol, refm, nerra)

        write (   6, 8538) xref, yref, zref
        write (iout, 8538) xref, yref, zref

        pdist = 0.0

        proxptx = xint
        proxpty = yint
        proxptz = zint

        proxvx  = 0.0
        proxvy  = 0.0
        proxvz  = 0.0

      elseif (nint .eq. 0) then       ! No intersection.

        write (   6, 8545)
        write (iout, 8545)

      else                            ! Vector is too short.

        write (   6, 8135)  ainput, ninput, aword(3)
        write (iout, 8135)  ainput, ninput, aword(3)

      endif                           ! Tested number of intersections.

c.... If no intersection, find smallest value of F(x,y,z) on track.
c....   This is where track is tangent to a surface F(x,y,z) = constant.
c....   Set (track vector) dot (normal vector) = 0.

 8550 format ('Smallest F at     ',1p3e20.12 /
     &        '  Track distance  ',1pe20.12 )

  540 if (nints .eq. 0) then
        q1 = qx(nq)  * xcom(nv) +
     &       qy(nq)  * ycom(nv) +
     &       qz(nq)  * zcom(nv)
        q2 = qxy(nq) * (xbeg * ycom(nv) + ybeg * xcom(nv)) +
     &       qyz(nq) * (ybeg * zcom(nv) + zbeg * ycom(nv)) +
     &       qzx(nq) * (zbeg * xcom(nv) + xbeg * zcom(nv))
        q3 = qxx(nq) * xbeg * xcom(nv) +
     &       qyy(nq) * ybeg * ycom(nv) +
     &       qzz(nq) * zbeg * zcom(nv)
        q4 = qxy(nq) * xcom(nv) * ycom(nv) +
     &       qyz(nq) * ycom(nv) * zcom(nv) +
     &       qzx(nq) * zcom(nv) * xcom(nv)
        q5 = qxx(nq) * xcom(nv)**2 +
     &       qyy(nq) * ycom(nv)**2 +
     &       qzz(nq) * zcom(nv)**2
        q6 = -(q1 + q2 + 2.0 * q3)
        q7 = 2.0 * (q4 + q5)
        if (q7 .ne. 0.0) then
          dmult = q6 / q7
          xmin  = xbeg + dmult * xcom(nv)
          ymin  = ybeg + dmult * ycom(nv)
          zmin  = zbeg + dmult * zcom(nv)
          dtrack = dmult * dlen
          write (   6, 8550) xmin, ymin, zmin, dtrack
          write (iout, 8550) xmin, ymin, zmin, dtrack
        endif
      endif

      go to 710

c===============================================================================

  610 if (ageomt .ne. 'zone') go to 700

c.... Input line:  track PNAME VNAME zone      ZNAME

 8620 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      do 690 nn = 1, nzquads(nz)   ! Loop over surfaces bounding zone.

c....   Find the index of the bounding surface.

        do 620 nq = 1, nquads
          if (aquad(nq) .eq. azquad(nn,n)) then
            nzq = nq
            go to 630
          endif
  620   continue
        write (   6, 8620) ainput, ninput, azquad(nn,n)
        write (iout, 8620) ainput, ninput, azquad(nn,n)
        go to 710

c....   Find the surface type (plane or non-planar quadric).

  630   if (aqtype(nzq) .ne. aqtypes(0)) go to 660

c....   Surface is a plane.  See if it is indeterminate.

        call plout (nzq, nzq, 'min')

        xn = qx(nzq)
        yn = qy(nzq)
        zn = qz(nzq)

        call aptvusz (xn, yn, zn, tol, dx, dy, dz, vlength)

        if (vlength .eq. 0.0) then
          write (   6, 8280) ainput, ninput, aword(5)
          write (iout, 8280) ainput, ninput, aword(5)
          go to 710
        endif

c....   Find the intersection point on the track, and the distance from the
c....     beginning of the track to the intersection point.

        call aptlnpl (xbeg, ybeg, zbeg, xend, yend, zend,
     &                xcen(nzq), ycen(nzq), zcen(nzq),
     &                xn, yn, zn, 1, tol,
     &                dperp, dtrack, fracab, xint, yint, zint,
     &                ipar, nerra)

        if (ipar .eq. 0) then         ! Not parallel.

          pdist = 0.0

          write (   6, 8400) xint, yint, zint  ! Intersection point.
          write (iout, 8400) xint, yint, zint
          call plotpt (xint, yint, zint, nerrg)

          write (   6, 8360) dtrack   ! Track distance to intersection.
          write (iout, 8360) dtrack

c....     Find the cosine of the angle of intersection.

          call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                  xn, yn, zn, 1, tol, vcos, nerra)

          if (dtrack .lt. 0.0) vcos = -vcos

c....     Find the angle of intersection.

          vang = acos (vcos)
          if (abs (vang) .lt. 1.e-5) vang = 0.0
          if (abs (vang - pi) .lt. 1.e-5) vang = pi

          if (angunit .eq. 'degrees') then
            vang = vang * 180.0 / pi
          endif

          write (   6, 8536) vcos, angunit, vang
          write (iout, 8536) vcos, angunit, vang

c....     Find the reflected track vector.

          xref = xcom(nv)
          yref = ycom(nv)
          zref = zcom(nv)

          call aptrefs (0, 0., 0., 0., xn, yn, zn, 0., 0., 0.,
     &                  xref, yref, zref, 1, tol, refm, nerra)

          write (   6, 8538) xref, yref, zref
          write (iout, 8538) xref, yref, zref

          proxptx = xint
          proxpty = yint
          proxptz = zint

          proxvx  = 0.0
          proxvy  = 0.0
          proxvz  = 0.0

        else                          ! Track parallel to plane.

          write (   6, 8480) dperp    ! Parallel distance.
          write (iout, 8480) dperp

          call aptptpl (xbeg, ybeg, zbeg,
     &                  xcen(nzq), ycen(nzq), zcen(nzq),
     &                  xn, yn, zn, 1, tol,
     &                  dpmin, cx, cy, cz, itrun, nerrpl)

          proxptx = xbeg
          proxpty = ybeg
          proxptz = zbeg

          proxvx  = cx - xbeg
          proxvx  = cy - ybeg
          proxvz  = cz - zbeg

        endif                         ! Tested for parallel track.

        go to 690

c....   Surface is a non-planar quadric.

c....   Find the intersection point on the track, and the distance from the
c....     beginning of the track to the intersection point.

  660   call quout (nzq, nzq, 'min', nerrq)

        tolp = tol * (abs (xbeg) + abs (ybeg) + abs (zbeg))
        if (tolp .eq. 0.0) tolp = tol

        dintmn = -1.e99
        dintmx =  -tolp

        call aptrkis (xbeg, ybeg, zbeg, xcom(nv), ycom(nv), zcom(nv),
     &                qc(nzq), qx(nzq), qy(nzq), qz(nzq),
     &                qxy(nzq), qyz(nzq), qzx(nzq),
     &                qxx(nzq), qyy(nzq), qzz(nzq),
     &                dintmn, dintmx, 1, tol,
     &                nint, xint, yint, zint, dtrack,
     &                snx, sny, snz, costh, nerra)

        if (nint .eq. 1) then         ! One intersection of track with quadric.

          write (   6, 8400) xint, yint, zint  ! Intersection point.
          write (iout, 8400) xint, yint, zint
          call plotpt (xint, yint, zint, nerrg)

          write (   6, 8360) dtrack   ! Track distance to intersection.
          write (iout, 8360) dtrack

c....     Find the normal vector at the intersection point.

          xnorm = qx(nzq) + 2.0 * qxx(nzq) * xint + qxy(nzq) * yint +
     &            qzx(nzq) * zint
          ynorm = qy(nzq) + qxy(nzq) * xint + 2.0 * qyy(nzq) * yint +
     &            qyz(nzq) * zint
          znorm = qz(nzq) + qzx(nzq) * xint + qyz(nzq) * yint +
     &          2.0 * qzz(nzq) * zint

          write (   6, 8534) xnorm, ynorm, znorm
          write (iout, 8534) xnorm, ynorm, znorm

          call aptvusz (xnorm, ynorm, znorm, tol, ux, uy, uz, vlength)

c....     Find the cosine of the angle of intersection.

          call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                  ux, uy, uz, 1, tol, vcos, nerra)

          if (dtrack .lt. 0.0) vcos = -vcos

c....     Find the angle of intersection.

          vang = acos (vcos)
          if (abs (vang) .lt. 1.e-5) vang = 0.0
          if (abs (vang - pi) .lt. 1.e-5) vang = pi

          if (angunit .eq. 'degrees') then
            vang = vang * 180.0 / pi
          endif

          if (vlength .eq. 0.0) then
            write (   6, 8535)
            write (iout, 8535)
            go to 670
          endif

          write (   6, 8536) vcos, angunit, vang
          write (iout, 8536) vcos, angunit, vang

c....     Find the reflected track vector.

          xref = xcom(nv)
          yref = ycom(nv)
          zref = zcom(nv)

          call aptrefs (0, 0., 0., 0., xnorm, ynorm, znorm, 0., 0., 0.,
     &                  xref, yref, zref, 1, tol, refm, nerra)

          write (   6, 8538) xref, yref, zref
          write (iout, 8538) xref, yref, zref

          pdist = 0.0

          proxptx = xint
          proxpty = yint
          proxptz = zint

          proxvx  = 0.0
          proxvy  = 0.0
          proxvz  = 0.0

        elseif (nint .eq. 0) then     ! No intersection.

          write (   6, 8532)
          write (iout, 8532)

        else                          ! Vector is too short.

          write (   6, 8135)  ainput, ninput, aword(3)
          write (iout, 8135)  ainput, ninput, aword(3)

        endif                         ! Tested number of intersections.

  670   dintmn = -tolp
        dintmx =  tolp

        call aptrkis (xbeg, ybeg, zbeg, xcom(nv), ycom(nv), zcom(nv),
     &                qc(nzq), qx(nzq), qy(nzq), qz(nzq),
     &                qxy(nzq), qyz(nzq), qzx(nzq),
     &                qxx(nzq), qyy(nzq), qzz(nzq),
     &                dintmn, dintmx, 1, tol,
     &                nint, xint, yint, zint, dtrack,
     &                snx, sny, snz, costh, nerra)

        if (nint .eq. 1) then         ! One intersection of track with quadric.
          write (   6, 8400) xint, yint, zint  ! Intersection point.
          write (iout, 8400) xint, yint, zint
          call plotpt (xint, yint, zint, nerrg)

          write (   6, 8360) dtrack   ! Track distance to intersection.
          write (iout, 8360) dtrack

c....     Find the normal vector at the intersection point.

          xnorm = qx(nzq) + 2.0 * qxx(nzq) * xint + qxy(nzq) * yint +
     &            qzx(nzq) * zint
          ynorm = qy(nzq) + qxy(nzq) * xint + 2.0 * qyy(nzq) * yint +
     &            qyz(nzq) * zint
          znorm = qz(nzq) + qzx(nzq) * xint + qyz(nzq) * yint +
     &          2.0 * qzz(nzq) * zint

          write (   6, 8534) xnorm, ynorm, znorm
          write (iout, 8534) xnorm, ynorm, znorm

          call aptvusz (xnorm, ynorm, znorm, tol, ux, uy, uz, vlength)

c....     Find the cosine of the angle of intersection.

          call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                  ux, uy, uz, 1, tol, vcos, nerra)

          if (dtrack .lt. 0.0) vcos = -vcos

c....     Find the angle of intersection.

          vang = acos (vcos)
          if (abs (vang) .lt. 1.e-5) vang = 0.0
          if (abs (vang - pi) .lt. 1.e-5) vang = pi

          if (angunit .eq. 'degrees') then
            vang = vang * 180.0 / pi
          endif

          if (vlength .eq. 0.0) then
            write (   6, 8535)
            write (iout, 8535)
            go to 680
          endif

          write (   6, 8536) vcos, angunit, vang
          write (iout, 8536) vcos, angunit, vang

c....     Find the reflected track vector.

          xref = xcom(nv)
          yref = ycom(nv)
          zref = zcom(nv)

          call aptrefs (0, 0., 0., 0., xnorm, ynorm, znorm, 0., 0., 0.,
     &                  xref, yref, zref, 1, tol, refm, nerra)

          write (   6, 8538) xref, yref, zref
          write (iout, 8538) xref, yref, zref

          pdist = 0.0

          proxptx = xint
          proxpty = yint
          proxptz = zint

          proxvx  = 0.0
          proxvy  = 0.0
          proxvz  = 0.0

        elseif (nint .eq. 0) then     ! No intersection.
          write (   6, 8542)
          write (iout, 8542)
        else                          ! Vector is too short.
          write (   6, 8135)  ainput, ninput, aword(3)
          write (iout, 8135)  ainput, ninput, aword(3)
        endif

  680   dintmn = tolp
        dintmx = 1.e99

        call aptrkis (xbeg, ybeg, zbeg, xcom(nv), ycom(nv), zcom(nv),
     &                qc(nzq), qx(nzq), qy(nzq), qz(nzq),
     &                qxy(nzq), qyz(nzq), qzx(nzq),
     &                qxx(nzq), qyy(nzq), qzz(nzq),
     &                dintmn, dintmx, 1, tol,
     &                nint, xint, yint, zint, dtrack,
     &                snx, sny, snz, costh, nerra)

        if (nint .eq. 1) then         ! One intersection of track with quadric.
          write (   6, 8400) xint, yint, zint  ! Intersection point.
          write (iout, 8400) xint, yint, zint
          call plotpt (xint, yint, zint, nerrg)

          write (   6, 8360) dtrack   ! Track distance to intersection.
          write (iout, 8360) dtrack

c....     Find the normal vector at the intersection point.

          xnorm = qx(nzq) + 2.0 * qxx(nzq) * xint + qxy(nzq) * yint +
     &            qzx(nzq) * zint
          ynorm = qy(nzq) + qxy(nzq) * xint + 2.0 * qyy(nzq) * yint +
     &            qyz(nzq) * zint
          znorm = qz(nzq) + qzx(nzq) * xint + qyz(nzq) * yint +
     &          2.0 * qzz(nzq) * zint

          write (   6, 8534) xnorm, ynorm, znorm
          write (iout, 8534) xnorm, ynorm, znorm

          call aptvusz (xnorm, ynorm, znorm, tol, ux, uy, uz, vlength)

c....     Find the cosine of the angle of intersection.

          call aptvang (xcom(nv), ycom(nv), zcom(nv),
     &                  ux, uy, uz, 1, tol, vcos, nerra)

          if (dtrack .lt. 0.0) vcos = -vcos

c....     Find the angle of intersection.

          vang = acos (vcos)
          if (abs (vang) .lt. 1.e-5) vang = 0.0
          if (abs (vang - pi) .lt. 1.e-5) vang = pi

          if (angunit .eq. 'degrees') then
            vang = vang * 180.0 / pi
          endif

          if (vlength .eq. 0.0) then
            write (   6, 8535)
            write (iout, 8535)
            go to 690
          endif

          write (   6, 8536) vcos, angunit, vang
          write (iout, 8536) vcos, angunit, vang

c....     Find the reflected track vector.

          xref = xcom(nv)
          yref = ycom(nv)
          zref = zcom(nv)

          call aptrefs (0, 0., 0., 0., xnorm, ynorm, znorm, 0., 0., 0.,
     &                  xref, yref, zref, 1, tol, refm, nerra)

          write (   6, 8538) xref, yref, zref
          write (iout, 8538) xref, yref, zref

          pdist = 0.0

          proxptx = xint
          proxpty = yint
          proxptz = zint

          proxvx  = 0.0
          proxvy  = 0.0
          proxvz  = 0.0

        elseif (nint .eq. 0) then     ! No intersection.

          write (   6, 8545)
          write (iout, 8545)

        else                          ! Vector is too short.

          write (   6, 8135)  ainput, ninput, aword(3)
          write (iout, 8135)  ainput, ninput, aword(3)

        endif

  690 continue                        ! End of loop over surfaces bounding zone.

      go to 710

c===============================================================================

c.... Input line:  unrecognized.

 8700 format ('  track ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Method not recognized:  object ',a)

  700 write (   6, 8700) ainput, ninput, aword(4)
      write (iout, 8700) ainput, ninput, aword(4)

c.... Switch back to the original coordinate system, if necessary.

  710 if (asysx .ne. 'rect') then
        nword = 2
        aword(2) = asysx
        call coord
      endif

  999 return

c.... End of subroutine track.        (+1 line)
      end
